//Module Imports
const { resolve: resolvePath } = require("path");
const { existsSync: fexists, unlinkSync: fremove } = require("fs");
const uuidGen = require("uuid-with-v6").v6;
//Local Imports
const storage = require("./storage");
const config = require("../config.json");

function load(uid) {
  return storage.getUserByCairoUuid(uid);
}

/**
 * Create a user with a uuid (should use Dunestorm API to login)
 */
function createUser(uuid) {
  storage.createUser(uuid);
}
/**
 * Creates file entry given aspects of a file updated
 */
function uploadFile(uuid, fileData) {
  return storage.uploadFile(uuid, fileData);
}
/**
 * Deletes files.
 * Requires Uuid to garuntee permission to delete a file
 * Sorts files by user before deleting to speed up reference updates
 */
function deleteFiles(uuid, targetFiles) {
  //Sort files by fileuuid to remove entries from the various users
  return storage.deleteFiles(uuid, targetFiles).then((deleteData) => {
    var files = deleteData.files;
    var deleteFails = deleteData.failed;
    files.forEach((file) => {
      try {
        fremove(file.path);
      } catch (e) {
        console.error("Error Deleting File", file.name, "\nPath:", file.path);
        deleteFails.push(`${file._id}`);
      }
    });
    return deleteFails.length > 0 && deleteFails;
  });
}
/**
 * Returns a list of fileUuids that the user owns
 */
function getOwnedFiles(userId) {
  return storage.getUserById(userId).then((user) => {
    const fileList = user.owned;
    var files = new Array(fileList.length);
    fileList.forEach(
      (file, i) =>
        (files[i] = new Promise((resolve, reject) =>
          storage.getFile(userId, file).then(resolve).catch(reject)
        ))
    );
    return Promise.all(files);
  });
}
/**
 * TODO: Impliment Advanced Sharing
 * Shares file with various people, and various permissions
 */
function shareFile(uuid, targetFile) {
  console.log(uuid, "requesting to share file");
  console.log(targetFile);
}
/**
 * TODO: Impliment Advanced Sharing
 * Returns all files shared with a user
 */
function getSharedFiles(uuid) {
  return storage.getSharedFileList(uuid);
}
/**
 * Checks if a the user is the owner and then toggles the list of files to public
 */
function publicfyFiles(uuid, files) {
  var publicfyFails = [];
  storage.publicfyFiles(uuid, files);
  return publicfyFails.length > 0 && publicfyFails;
}
module.exports = {
  createUser,
  uploadFile,
  deleteFiles,
  getOwnedFiles,
  publicfyFiles,
  shareFile,
  getSharedFiles,
  load,
};
